import 'package:flutter/material.dart';
import 'package:url_launcher/url_launcher.dart';
import 'package:natinfo_flutter/pages/natinf_home_page.dart';
import 'package:natinfo_flutter/repositories/natinf_repository.dart';

class DataInitializationScreen extends StatefulWidget {
  final NatinfRepository repository;
  const DataInitializationScreen({super.key, required this.repository});

  @override
  _DataInitializationScreenState createState() => _DataInitializationScreenState();
}

class _DataInitializationScreenState extends State<DataInitializationScreen> {
  String _status = "Vérification des données locales...";
  double _progress = 0.0;
  int _phase = 0; // 0: aucune phase, 1: téléchargement, 2: importation

  @override
  void initState() {
    super.initState();
    _checkAndFetch();
  }

  Future<bool> _promptDownloadConfirmation(int downloadSize) async {
    return await showDialog<bool>(
          context: context,
          builder: (context) {
            return AlertDialog(
              title: const Text("Téléchargement requis"),
              content: Text(
                "Le téléchargement initial nécessite environ $downloadSize Mo de ressources supplémentaires.\n\nVoulez-vous procéder au téléchargement ?",
              ),
              actions: [
                TextButton(
                  onPressed: () => Navigator.of(context).pop(false),
                  child: const Text("Annuler"),
                ),
                TextButton(
                  onPressed: () => Navigator.of(context).pop(true),
                  child: const Text("Télécharger"),
                ),
              ],
            );
          },
        ) ??
        false;
  }

  Future<void> _checkAndFetch() async {
    try {
      final hasData = await widget.repository.hasLocalData();
      if (!hasData) {
        const int downloadSize = 15;

        final confirmed = await _promptDownloadConfirmation(downloadSize);
        if (!confirmed) {
          setState(() {
            _status = "Téléchargement annulé par l'utilisateur.";
          });
          return;
        }

        setState(() {
          _status = "Préparation du téléchargement...";
          _progress = 0.0;
          _phase = 0;
        });
        await widget.repository.fetchInitialData(
          onStatus: (msg) {
            setState(() {
              _status = msg;
            });
          },
          onProgress: (progressFraction) {
            setState(() {
              _progress = progressFraction;
            });
          },
          onPhase: (phase) {
            setState(() {
              _phase = phase;
            });
          },
        );
      }
      Navigator.pushReplacement(
        context,
        MaterialPageRoute(builder: (context) => const NatinfHomePage()),
      );
    } catch (e) {
      setState(() {
        _status = "Erreur lors du téléchargement\nVérifiez votre connexion Internet.";
      });
    }
  }

  void _retry() {
    setState(() {
      _status = "Réinitialisation et nouvelle tentative...";
      _progress = 0.0;
      _phase = 0;
    });
    _checkAndFetch();
  }

  Future<void> _launchIssueUrl() async {
    final Uri url = Uri.parse("https://codeberg.org/retiolus/natinfo_flutter/issues");
    if (await canLaunchUrl(url)) {
      await launchUrl(url, mode: LaunchMode.externalApplication);
    }
  }

  @override
  Widget build(BuildContext context) {
    final bool hasError = _status.startsWith("Erreur") || _status.startsWith("Téléchargement annulé");
    Widget content;
    if (hasError) {
      content = Column(
        mainAxisAlignment: MainAxisAlignment.center,
        children: [
          Icon(Icons.error, size: 48, color: Colors.red.shade300),
          const SizedBox(height: 16),
          Text(
            _status,
            textAlign: TextAlign.center,
            style: const TextStyle(fontSize: 16),
          ),
          const SizedBox(height: 16),
          ElevatedButton(
            onPressed: _retry,
            child: const Text("Réessayer"),
          ),
          const SizedBox(height: 16),
          GestureDetector(
            onTap: _launchIssueUrl,
            child: Text(
              "Ouvrir une issue sur Codeberg",
              style: TextStyle(
                decoration: TextDecoration.underline,
                color: Theme.of(context).colorScheme.secondary,
            ),
          ),
        ),
        ],
      );
    } else {
      Widget progressWidget;
      if (_phase == 1) {
        progressWidget = const CircularProgressIndicator();
      } else if (_phase == 2) {
        progressWidget = LinearProgressIndicator(
          value: _progress,
          minHeight: 8,
          backgroundColor: Colors.grey.shade300,
          valueColor: AlwaysStoppedAnimation<Color>(Theme.of(context).primaryColor),
        );
      } else {
        progressWidget = const CircularProgressIndicator();
      }
      content = Column(
        mainAxisAlignment: MainAxisAlignment.center,
        children: [
          progressWidget,
          const SizedBox(height: 16),
          Text(_status, textAlign: TextAlign.center),
        ],
      );
    }

    return Scaffold(
      body: Center(
        child: Padding(
          padding: const EdgeInsets.symmetric(horizontal: 24),
          child: content,
        ),
      ),
      bottomNavigationBar: Padding(
        padding: const EdgeInsets.symmetric(vertical: 8.0, horizontal: 16.0),
        child: Column(
          mainAxisSize: MainAxisSize.min,
          children: [
            Text(
              "NATINFo - Développé par retiolus",
              textAlign: TextAlign.center,
              style: TextStyle(
                fontSize: 12,
                color: Colors.grey.shade700,
                fontStyle: FontStyle.italic,
              ),
            ),
            const SizedBox(height: 4),
            Text(
              "Cette application n'est pas un service officiel de l'État français et n'est ni affiliée, ni soutenue par l'administration publique.",
              textAlign: TextAlign.center,
              style: TextStyle(
                fontSize: 12,
                color: Colors.grey.shade600,
                fontStyle: FontStyle.italic,
              ),
            ),
          ],
        ),
      ),
    );
  }
}
