import 'package:flutter/material.dart';
import 'package:natinfo_flutter/api/swagger.swagger.dart';
import 'package:natinfo_flutter/db/database_helper.dart';
import 'package:natinfo_flutter/widgets/natinf_list.dart';

enum SortOption {
  ascendingNum,
  descendingNum,
  newestFirst,
  oldestFirst,
  mostConsulted,
  leastConsulted,
}

class FavouritesPage extends StatefulWidget {
  const FavouritesPage({super.key});

  @override
  _FavouritesPageState createState() => _FavouritesPageState();
}

class _FavouritesPageState extends State<FavouritesPage> {
  List<Map<String, dynamic>> _favouritesRecords = [];
  List<Map<String, dynamic>> _historyRecords = [];
  List<Natinf> _natinfFavourites = [];
  SortOption _sortOption = SortOption.mostConsulted;
  bool _isLoading = true;

  @override
  void initState() {
    super.initState();
    _loadFavourites();
  }

  Future<void> _loadFavourites() async {
    final records = await DatabaseHelper().getFavourites();
    final history_records = await DatabaseHelper().getHistory();
    final futures = records.map((rec) async {
      return await DatabaseHelper().getNatinfByNumero(rec['numero_natinf']);
    }).toList();
    final results = await Future.wait(futures);
    setState(() {
      _favouritesRecords = records;
      _historyRecords = history_records;
      _natinfFavourites = results.whereType<Natinf>().toList();
      _isLoading = false;
    });
  }

  Future<void> _confirmClearFavourites() async {
    final shouldClear = await showDialog<bool>(
      context: context,
      builder: (context) => AlertDialog(
        title: const Text("Confirmer la suppression"),
        content: const Text("Voulez-vous vraiment supprimer les favoris ?"),
        actions: [
          TextButton(
            onPressed: () => Navigator.pop(context, false),
            child: const Text("Annuler"),
          ),
          TextButton(
            onPressed: () => Navigator.pop(context, true),
            child: const Text("Supprimer"),
          ),
        ],
      ),
    );
    if (shouldClear == true) {
      await DatabaseHelper().clearFavourites();
      setState(() {
        _favouritesRecords.clear();
        _natinfFavourites.clear();
      });
    }
  }

  List<Natinf> _applySort() {
    switch (_sortOption) {
      case SortOption.ascendingNum:
        var list = List<Natinf>.from(_natinfFavourites);
        list.sort((a, b) => (a.numeroNatinf ?? '').compareTo(b.numeroNatinf ?? ''));
        return list;
      case SortOption.descendingNum:
        var list = List<Natinf>.from(_natinfFavourites);
        list.sort((a, b) => (b.numeroNatinf ?? '').compareTo(a.numeroNatinf ?? ''));
        return list;
      case SortOption.newestFirst:
        var list = List<Natinf>.from(_natinfFavourites);
        final addedTimes = <String, int>{};
        for (var rec in _favouritesRecords) {
          addedTimes[rec['numero_natinf'].toString()] = rec['added_at'] as int;
        }
        list.sort((a, b) {
          final timeA = addedTimes[a.numeroNatinf ?? ''] ?? 0;
          final timeB = addedTimes[b.numeroNatinf ?? ''] ?? 0;
          return timeB.compareTo(timeA);
        });
        return list;
      case SortOption.oldestFirst:
        var list = List<Natinf>.from(_natinfFavourites);
        final addedTimes = <String, int>{};
        for (var rec in _favouritesRecords) {
          addedTimes[rec['numero_natinf'].toString()] = rec['added_at'] as int;
        }
        list.sort((a, b) {
          final timeA = addedTimes[a.numeroNatinf ?? ''] ?? 0;
          final timeB = addedTimes[b.numeroNatinf ?? ''] ?? 0;
          return timeA.compareTo(timeB);
        });
        return list;
      case SortOption.mostConsulted:
      case SortOption.leastConsulted:
        final counts = <String, int>{};
        for (var rec in _historyRecords) {
          final key = rec['numero_natinf'].toString();
          counts[key] = (counts[key] ?? 0) + 1;
        }
        final mapN = <String, Natinf>{};
        for (var n in _natinfFavourites) {
          final key = n.numeroNatinf?.toString() ?? '';
          mapN[key] = n;
        }
        final entries = counts.entries.toList();
        entries.sort((a, b) {
          if (_sortOption == SortOption.mostConsulted) {
            return b.value.compareTo(a.value);
          } else {
            return a.value.compareTo(b.value);
          }
        });
        return entries
            .where((e) => mapN.containsKey(e.key))
            .map((e) => mapN[e.key]!)
            .toList();
      default:
        return List.from(_natinfFavourites);
    }
  }

  @override
  Widget build(BuildContext context) {
    final displayList = _applySort();
    return Scaffold(
      appBar: AppBar(
        title: const Text("Favoris"),
        actions: [
          IconButton(
            icon: const Icon(Icons.delete),
            onPressed: _confirmClearFavourites,
          )
        ],
      ),
      body: _isLoading
          ? const Center(child: CircularProgressIndicator())
          : Padding(
              padding: const EdgeInsets.all(16),
              child: Column(
                crossAxisAlignment: CrossAxisAlignment.start,
                children: [
                  DropdownButtonFormField<SortOption>(
                    value: _sortOption,
                    decoration: InputDecoration(
                      labelText: 'Trier par',
                      border: OutlineInputBorder(borderRadius: BorderRadius.circular(8)),
                      contentPadding: EdgeInsets.symmetric(horizontal: 12, vertical: 4),
                    ),
                    onChanged: (val) {
                      if (val != null) setState(() => _sortOption = val);
                    },
                    items: [
                      DropdownMenuItem(
                        value: SortOption.ascendingNum,
                        child: Text('Numéro croissant'),
                      ),
                      DropdownMenuItem(
                        value: SortOption.descendingNum,
                        child: Text('Numéro décroissant'),
                      ),
                      DropdownMenuItem(
                        value: SortOption.newestFirst,
                        child: Text('Les plus récents en premier'),
                      ),
                      DropdownMenuItem(
                        value: SortOption.oldestFirst,
                        child: Text('Les plus anciens en premier'),
                      ),
                      DropdownMenuItem(
                        value: SortOption.mostConsulted,
                        child: Text('Les plus consultés'),
                      ),
                      DropdownMenuItem(
                        value: SortOption.leastConsulted,
                        child: Text('Les moins consultés'),
                      ),
                    ],
                  ),
                  const SizedBox(height: 8),
                  Expanded(
                    child: Container(
                      margin: const EdgeInsets.only(top: 8.0),
                      decoration: BoxDecoration(
                        borderRadius: BorderRadius.circular(8),
                      ),
                      child: ClipRRect(
                        borderRadius: BorderRadius.circular(8),
                        child: NatinfList(
                          natinfList: displayList,
                        ),
                      ),
                    ),
                  ),
                ],
              ),
            ),
    );
  }
}
