import 'package:flutter/material.dart';
import 'package:natinfo_flutter/providers/theme_provider.dart';
import 'package:provider/provider.dart';
import 'package:url_launcher/url_launcher.dart';
import 'package:natinfo_flutter/api/swagger.swagger.dart';
import 'package:natinfo_flutter/db/database_helper.dart';

class NatinfDetail extends StatefulWidget {
  final Natinf natinf;
  const NatinfDetail({super.key, required this.natinf});

  @override
  State<NatinfDetail> createState() => _NatinfDetailState();
}

class _NatinfDetailState extends State<NatinfDetail> {
  bool _isFavourite = false;

  @override
  void initState() {
    super.initState();
    _checkFavouriteStatus();
  }

  Future<void> _checkFavouriteStatus() async {
    if (widget.natinf.numeroNatinf != null) {
      final isFav = await DatabaseHelper().isFavourite(widget.natinf.numeroNatinf!);
      setState(() {
        _isFavourite = isFav;
      });
    }
  }

  Future<void> _toggleFavourite() async {
    if (widget.natinf.numeroNatinf == null) return;
    if (_isFavourite) {
      await DatabaseHelper().removeFavourite(widget.natinf.numeroNatinf!);
            ScaffoldMessenger.of(context).showSnackBar(
        const SnackBar(content: Text("Supprimé des favoris")),
            );
    } else {
      await DatabaseHelper().addFavourites(widget.natinf.numeroNatinf!);
                ScaffoldMessenger.of(context).showSnackBar(
                  const SnackBar(content: Text("Ajouté aux favoris")),
                );
              }
    setState(() {
      _isFavourite = !_isFavourite;
    });
  }

  Color getBadgeColor(BuildContext context, String text) {
    final lower = text.trim().toLowerCase();
    final themeProvider = Provider.of<ThemeProvider>(context, listen: false);
    if (lower == 'obligatoire') return themeProvider.obligatoryColor;
    if (lower == 'facultatif') return themeProvider.facultativeColor;
    if (lower == 'non') return themeProvider.nonColor;
    if (lower.contains('crime')) return themeProvider.crimeColor;
    if (lower.contains('delit') || lower.contains('délit')) return themeProvider.delitColor;
    if (lower.contains('contravention')) return themeProvider.contraventionColor;
    return Colors.transparent;
  }

  bool hasBadge(String text) {
    final lower = text.trim().toLowerCase();
    return lower == 'obligatoire' ||
        lower == 'facultatif' ||
        lower == 'non' ||
        lower.contains('crime') ||
        lower.contains('delit') ||
        lower.contains('délit') ||
        lower.contains('contravention');
  }

  Widget buildBadge(String text) {
    if (!hasBadge(text)) return Text(text);
    return IntrinsicWidth(
      child: Container(
        padding: const EdgeInsets.symmetric(horizontal: 8, vertical: 4),
        decoration: BoxDecoration(
          color: getBadgeColor(context, text),
          borderRadius: BorderRadius.circular(12),
        ),
        child: Text(
          text,
        ),
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    final List<Widget> normalTiles = [];
    final List<Widget> bottomTiles = [];
    void addTile(String title, String? value, {bool useBadge = true}) {
      if (value == null || value.isEmpty || value == "N/A") return;
      final tile = ListTile(
        title: Text(
          title,
          style: const TextStyle(fontWeight: FontWeight.bold),
        ),
        subtitle: useBadge ? buildBadge(value) : Text(value),
      );
      if (value == "non") {
        bottomTiles.add(tile);
      } else {
        normalTiles.add(tile);
      }
    }

    String? modesProcesVerbalSubtitle;
    if (widget.natinf.modesProcesVerbal != null) {
      if (widget.natinf.modesProcesVerbal is List) {
        modesProcesVerbalSubtitle = (widget.natinf.modesProcesVerbal as List<dynamic>)
            .map((mode) {
          switch (mode) {
            case 0:
              return 'CRI';
            case 1:
              return 'PVE';
            case 2:
              return 'PVO';
            default:
              return mode.toString();
          }
        }).join(' ou ');
      } else {
        modesProcesVerbalSubtitle = widget.natinf.modesProcesVerbal.toString();
      }
    }

    addTile("Qualification de l'infraction", widget.natinf.qualificationInfraction, useBadge: false);
    addTile("Nature de l'infraction", widget.natinf.natureInfraction);
    addTile("Définie par", widget.natinf.definiePar);
    addTile("Réprimée par", widget.natinf.reprimeePar);
    addTile("Mode de rédaction", modesProcesVerbalSubtitle);
    addTile("Retrait points", widget.natinf.retraitPoints?.toString());
    addTile("Contrôle alcoolémie", widget.natinf.controleAlcoolemie?.value);
    addTile("Dépistage imprégnation alcoolique", widget.natinf.depistageImpregnationAlcoolique?.value);
    addTile("Dépistage stupéfiants", widget.natinf.depistageStupefiants?.value);
    addTile("Rétention permis de conduire", widget.natinf.retentionPermisConduire?.value);
    addTile("Immobilisation du véhicule", widget.natinf.immobilisationVehicule?.value);
    addTile("Mise en fourrière", widget.natinf.miseEnFourriere?.value);
    addTile("Observations", widget.natinf.observations, useBadge: false);

    final List<Widget> allTiles = [...normalTiles]..addAll(bottomTiles);

    final String webUrl = "https://natinfo.retiolus.net/natinf/${widget.natinf.numeroNatinf}";
    allTiles.add(
      ListTile(
        title: const Text(
          "Accéder à plus de détails en ligne",
          style: TextStyle(
            color: Colors.blue,
            decoration: TextDecoration.underline,
          ),
        ),
        onTap: () async {
          final Uri url = Uri.parse(webUrl);
          if (await canLaunchUrl(url)) {
            await launchUrl(url, mode: LaunchMode.externalApplication);
          } else {
            ScaffoldMessenger.of(context).showSnackBar(
              const SnackBar(content: Text("Impossible d'ouvrir le lien")),
            );
          }
        },
      ),
    );

    return Scaffold(
      appBar: AppBar(
        title: Text("Détails du NATINF ${widget.natinf.numeroNatinf ?? "N/A"}"),
        actions: [
          IconButton(
            icon: Icon(_isFavourite ? Icons.star : Icons.star_border),
            tooltip: _isFavourite ? 'Supprimer des favoris' : 'Ajouter aux favoris',
            onPressed: _toggleFavourite,
          ),
        ],
      ),
      body: Padding(
        padding: const EdgeInsets.all(16.0),
        child: ListView(
          children: allTiles,
        ),
      ),
    );
  }
}
