import 'package:flutter/material.dart';
import 'package:natinfo_flutter/api/swagger.swagger.dart';
import 'package:natinfo_flutter/config/app_config.dart';
import 'package:natinfo_flutter/providers/operational_mode_provider.dart';
import 'package:natinfo_flutter/widgets/app_drawer.dart';
import 'package:natinfo_flutter/widgets/natinf_list.dart';
import 'package:provider/provider.dart';
import 'package:natinfo_flutter/repositories/natinf_repository.dart';

class NatinfHomePage extends StatefulWidget {
  const NatinfHomePage({super.key});

  @override
  _NatinfHomePageState createState() => _NatinfHomePageState();
}

class _NatinfHomePageState extends State<NatinfHomePage> {
  final TextEditingController _controller = TextEditingController();
  List<Natinf>? _results;
  bool _isLoading = false;
  String _errorMessage = '';
  String _statusMessage = '';

  Future<void> _search() async {
    FocusScope.of(context).unfocus();
    setState(() {
      _isLoading = true;
      _errorMessage = '';
      _results = null;
      _statusMessage = "Démarrage de la recherche...";
    });
    final repository = Provider.of<NatinfRepository>(context, listen: false);
    try {
      final results = await repository.searchNatinf(
        _controller.text,
        onStatus: (msg) {
          setState(() {
            _statusMessage = msg;
          });
        },
        forceOffline: AppConfig.forceOffline,
      );
      setState(() {
        _results = results;
      });
    } catch (e) {
      setState(() {
        _errorMessage = e.toString();
      });
    } finally {
      setState(() {
        _isLoading = false;
      });
    }
  }

  Widget _buildSearchBar() {
    return Row(
      children: [
        Expanded(
          child: TextField(
            controller: _controller,
            decoration: const InputDecoration(
              labelText: "Recherche d'infractions",
              hintText: 'ex: numéro, nature, qualification, articles...',
              border: OutlineInputBorder(),
            ),
          ),
        ),
        const SizedBox(width: 8),
        ElevatedButton(
          onPressed: _isLoading ? null : _search,
          style: ElevatedButton.styleFrom(
            padding: const EdgeInsets.symmetric(horizontal: 16, vertical: 20),
          ),
          child: _isLoading
              ? const SizedBox(
                  width: 24,
                  height: 24,
                  child: CircularProgressIndicator(
                    strokeWidth: 2.5,
                    backgroundColor: Colors.white,
                  ),
                )
              : const Icon(Icons.search),
        ),
      ],
    );
  }

  @override
  Widget build(BuildContext context) {
    final opMode = context.watch<OperationalModeProvider>().opMode;

    final disclaimer = Padding(
      padding: const EdgeInsets.symmetric(vertical: 8.0, horizontal: 16.0),
      child: Text(
        "Cette application n'est pas un service officiel de l'État français et n'est ni affiliée, ni soutenue par l'administration publique.",
        textAlign: TextAlign.center,
        style: TextStyle(
          fontSize: 12,
          color: Colors.grey.shade600,
          fontStyle: FontStyle.italic,
        ),
      ),
    );

    return Scaffold(
      appBar: AppBar(title: const Text('NATINFo')),
      drawer: const AppDrawer(),
      body: Padding(
        padding: const EdgeInsets.all(16),
              child: Column(
                children: [
            if (!opMode) _buildSearchBar(),
            if (!opMode) const SizedBox(height: 16),
            if (_statusMessage.isNotEmpty)
              Padding(
                padding: const EdgeInsets.only(bottom: 8.0),
                child: Text(
                  _statusMessage,
                  style: const TextStyle(
                    fontStyle: FontStyle.italic,
                    color: Colors.blueGrey,
                  ),
              ),
              ),
            if (_errorMessage.isNotEmpty)
              Text(
                _errorMessage,
                style: const TextStyle(color: Colors.red),
              ),
            Expanded(
              child: _results != null
                  ? NatinfList(natinfList: _results!)
                  : Container(),
            ),
          ],
        ),
      ),
      bottomNavigationBar: opMode
          ? SafeArea(
              child: Column(
                mainAxisSize: MainAxisSize.min,
                children: [
                  Padding(
                    padding: const EdgeInsets.symmetric(horizontal: 16.0),
                    child: _buildSearchBar(),
                  ),
                  disclaimer,
                ],
              ),
            )
          : disclaimer,
    );
  }
}
